package lightJackson;

import java.io.IOException;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import com.fasterxml.jackson.jr.annotationsupport.JacksonAnnotationExtension;
import com.fasterxml.jackson.jr.ob.JSON;

import lightJackson.data.ComplexJsonData;

class ComplexJsonTest {

	String json;

	@BeforeEach
	void setup() {
		try {
			json = JSON.std.with(JSON.Feature.PRETTY_PRINT_OUTPUT).composeString().startObject()
					.startArrayField("persons").startObject().put("name", "name1").put("age", 11).end().startObject()
					.put("name", "name2").put("age", 12).end().end().startArrayField("numbers").add(1).add(2).add(3)
					.end().put("someName", "Test").end().finish();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	@Test
	void test_composejson_for_complex() {
		try {
			System.out.println("Original Json:\n" + json);

			// De-serialization
			ComplexJsonData complexDataObj = JSON.std.with(JSON.Feature.PRETTY_PRINT_OUTPUT)
					.beanFrom(ComplexJsonData.class, json);

			// Serialization
			String newJson = JSON.std
					.with(JSON.Feature.PRETTY_PRINT_OUTPUT, JSON.Feature.WRITE_NULL_PROPERTIES,
							JSON.Feature.PRESERVE_FIELD_ORDERING, JSON.Feature.FAIL_ON_DUPLICATE_MAP_KEYS)
					.asString(complexDataObj);
			System.out.println("Serialization Json:\n" + newJson);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	@Test
	void test_json_to_complex_keepOrder() {
		try {
			System.out.println("Original Json:\n" + json);

			// De-serialization
			ComplexJsonData complexDataObj = JSON.std.with(JSON.Feature.PRETTY_PRINT_OUTPUT)
					.beanFrom(ComplexJsonData.class, json);

			// Serialization
			String newJson = JSON.builder().register(JacksonAnnotationExtension.std).build()
					.with(JSON.Feature.PRETTY_PRINT_OUTPUT, JSON.Feature.WRITE_NULL_PROPERTIES,
							JSON.Feature.PRESERVE_FIELD_ORDERING, JSON.Feature.FAIL_ON_DUPLICATE_MAP_KEYS)
					.asString(complexDataObj);
			System.out.println("Serialization Json:\n" + newJson);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

}