package lightJackson;

import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertTrue;

import java.io.IOException;
import java.time.LocalDate;
import java.time.LocalDateTime;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import com.fasterxml.jackson.jr.annotationsupport.JacksonAnnotationExtension;
import com.fasterxml.jackson.jr.ob.JSON;
import com.fasterxml.jackson.jr.ob.JacksonJrExtension;
import com.fasterxml.jackson.jr.ob.api.ExtensionContext;

import lightJackson.data.Person;

class SerializePersonTest {

	private Person person;

	@BeforeEach
	void setup() {
		person = new Person("Zheng", 30);
		person.setOverrideName("Mary");
		person.setEmail("test@test.com");
		person.setIgnoredElement("should be ignored");
		person.setBirthDate(LocalDate.now());
	}

	@Test
	void test_serialize_json_annotation_customize() {
		JSON annotationCustomizeMapper = JSON.builder().register(JacksonAnnotationExtension.std)
				.register(new JacksonJrExtension() {
					@Override
					protected void register(ExtensionContext extensionContext) {
						extensionContext.insertProvider(new MyHandlerProvider());
					}

				}).build().with(JSON.Feature.PRETTY_PRINT_OUTPUT);

		String json;
		try {
			person.setBirthDate(LocalDate.now());
			json = annotationCustomizeMapper.asString(person);
			assertTrue(json.contains("override_name"), "it should contain override_name elements");
			assertTrue(json.contains("2024-05-17"), "It should display the date as yyyy-mm-dd format");

			System.out.println("JSON from annotCustMapper:\n" + json);
		} catch (IOException e) {
			e.printStackTrace();
		}

	}

	@Test
	void test_serialize_json_annotation() {
		try {
			JSON annotationMapper = JSON.builder().register(JacksonAnnotationExtension.std).build()
					.with(JSON.Feature.PRETTY_PRINT_OUTPUT, JSON.Feature.WRITE_NULL_PROPERTIES);
			person.setBirthDate(null);
			String json = annotationMapper.asString(person);
			assertTrue(json.contains("override_name"), "it should contain override_name elements");
			assertFalse(json.contains("should be ignored"), "it should be ignored");

			System.out.println("JSON from annotationMapper:\n" + json);

		} catch (IOException e) {
			e.printStackTrace();
		}

	}

	@Test
	void test_serialize_json_custDate() {
		try {
			person.setLogTimeStamp(LocalDateTime.now());

			JSON customizeMapper = JSON.builder().register(new JacksonJrExtension() {
				@Override
				protected void register(ExtensionContext extensionContext) {
					extensionContext.insertProvider(new MyHandlerProvider());
				}

			}).build().with(JSON.Feature.PRETTY_PRINT_OUTPUT, JSON.Feature.WRITE_NULL_PROPERTIES);

			String json = customizeMapper.asString(person);
			assertTrue(json.contains("2024-05-17"), "It should display the date as yyyy-mm-dd format");

			System.out.println("JSON from customizeMapper:\n" + json);

		} catch (IOException e) {
			e.printStackTrace();
		}

	}

	@Test
	void test_serialize_json_pretty() {
		try {
			JSON prettyMapper = JSON.std.with(JSON.Feature.PRETTY_PRINT_OUTPUT);

			String json = prettyMapper.asString(person);
			assertFalse(json.contains("null"), "it should NOT contain null elements");
			assertTrue(json.contains("should be ignored"), "it should NOT be ignored without annotation");
			assertFalse(json.contains("override_Name"), "it should NOT contain override_Name elements");
			System.out.println("JSON from prettyMapper:\n" + json);

		} catch (IOException e) {
			e.printStackTrace();
		}

	}

	@Test
	void test_serialize_json_withNull() {
		try {
			JSON prettyMapperWithNull = JSON.std.with(JSON.Feature.PRETTY_PRINT_OUTPUT,
					JSON.Feature.WRITE_NULL_PROPERTIES, JSON.Feature.FAIL_ON_DUPLICATE_MAP_KEYS,
					JSON.Feature.PRESERVE_FIELD_ORDERING);

			String json = prettyMapperWithNull.asString(person);
			assertTrue(json.contains("null"), "it should contain null elements");
			System.out.println("JSON from prettyMapperWithNull:\n" + json);

		} catch (IOException e) {
			e.printStackTrace();
		}
	}

}
